'''
common definitions used across the GUI
'''

import os.path

# Directories to save things in. Will have a sub folder for each measurement
save_dir = os.path.join(os.path.expanduser("~"), 'Documents', 'MCV-QoE')
old_save_dir = os.path.join(os.path.expanduser("~"), 'MCV-QoE')